/**
 * Copyright (c) 2013-present, Facebook, Inc. All rights reserved.
 *
 * This file provided by Facebook is for non-commercial testing and evaluation
 * purposes only. Facebook reserves all rights not expressly granted.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * FACEBOOK BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

'use strict';

exports.__esModule = true;
exports.TextBlock = undefined;

var _class, _temp;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _Chip = require('material-ui/Chip');

var _Chip2 = _interopRequireDefault(_Chip);

var _withStyles = require('material-ui/styles/withStyles');

var _withStyles2 = _interopRequireDefault(_withStyles);

var _CodeBlock = require('./CodeBlock');

var _CodeBlock2 = _interopRequireDefault(_CodeBlock);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var styles = function styles(theme) {

  // console.log("theme", theme);

  if (!theme) {
    return {};
  }

  return {

    chip: {
      margin: theme.spacing.unit,
      height: 25,
      cursor: 'pointer',

      '&.active': {
        backgroundColor: theme.palette.primary[500],
        color: "white",

        '& .label': {
          color: theme.palette.secondary.contrastDefaultColor
        }
      }
    },
    svgIcon: {},
    row: {
      display: 'flex',
      justifyContent: 'center',
      flexWrap: 'wrap'
    }
  };
};

var TextBlock = exports.TextBlock = (_temp = _class = function (_React$Component) {
  _inherits(TextBlock, _React$Component);

  function TextBlock(props) {
    _classCallCheck(this, TextBlock);

    // console.log('TextBlock construct', props);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

    _this.state = {
      editMode: false,
      lang: _this._getLang() || props.lang,
      allow_edit: props.blockProps.allow_edit
    };

    // console.log('TextBlock construct state', props, this.state);

    _this._onClick = function () {

      // alert(!this.state.allow_edit);
      // alert(this.state.editMode);

      if (!_this.state.allow_edit) {
        return;
      }

      if (_this.state.editMode) {
        return;
      }

      _this.setState({
        editMode: true,
        textValue: _this._getValue()
      }, function () {
        _this._startEdit();
      });
    };

    _this._onValueChange = function (evt) {
      var value = evt.target.value;
      _this.setState({
        textValue: value
      });
    };

    /*
    *   Сохранение.
    *   Если контента нет, то удаляем этот блок
    * */
    _this._save = function () {

      var value = _this.state.textValue;

      if (value !== '') {
        var entityKey = _this.props.block.getEntityAt(0);
        var newContentState = _this.props.contentState.mergeEntityData(entityKey, {
          content: _this.state.textValue,
          lang: _this.state.lang
        });
        _this.setState({
          invalidTeX: false,
          editMode: false,
          textValue: null
        }, _this._finishEdit.bind(_this, newContentState));
      } else {
        _this._remove();
      }
    };

    _this._remove = function () {

      // console.log('this.props.blockProps.onRemove', this.props.block.getKey());

      _this.props.blockProps.onRemove(_this.props.block.getKey());
    };
    _this._startEdit = function () {
      _this.props.blockProps.onStartEdit(_this.props.block.getKey());
    };
    _this._finishEdit = function (newContentState) {
      _this.props.blockProps.onFinishEdit(_this.props.block.getKey(), newContentState);
    };
    return _this;
  }

  TextBlock.prototype._getData = function _getData() {
    var data = this.props.contentState.getEntity(this.props.block.getEntityAt(0)).getData();

    return data;
  };

  TextBlock.prototype._getValue = function _getValue() {
    var content = this._getData()['content'];

    // if(
    //   this.state.editMode &&
    //   content == 'Кликните чтобы начать редактирование'
    // ){
    //   content = '';
    // }

    return content;
  };

  TextBlock.prototype._getLang = function _getLang() {
    var lang = this._getData()['lang'];

    // if(
    //   this.state.editMode &&
    //   content == 'Кликните чтобы начать редактирование'
    // ){
    //   content = '';
    // }

    return lang;
  };

  // _onTab(event) {
  //   this._onChange(RichUtils.onTab(event, this.state.editorState, 6));
  // }

  TextBlock.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {

    // console.log("TextBlock componentWillReceiveProps", nextProps);

    if (nextProps.blockProps.allow_edit != this.state.allow_edit) {
      this.setState({
        allow_edit: nextProps.blockProps.allow_edit
      });
    }

    return true;
  };

  TextBlock.prototype.render = function render() {
    var _this2 = this;

    var classes = this.props.classes;


    var texContent = null;
    if (this.state.editMode) {
      // if (this.state.invalidTeX) {
      //   texContent = '';
      // } else {
      //   texContent = this.state.textValue;
      // }

      texContent = this.state.textValue;

      // if(texContent == 'Кликните чтобы начать редактирование'){
      //   texContent = '';
      // }
    } else {

      if (this.props.blockProps.fullView === false) {
        return null;
      }

      texContent = this._getValue();
    }

    var className = 'Editor-text';
    if (this.state.editMode) {
      className += ' Editor-activeText';
    }

    var editPanel = null;
    // if (this.state.allow_edit && this.state.editMode) {
    if (this.state.editMode) {
      var buttonClass = 'Editor-saveButton';
      // if (this.state.invalidTeX) {
      //   buttonClass += ' Editor-invalidButton';
      // }

      var height = "40px";

      var langs = [{
        value: "php",
        label: "PHP"
      }, {
        value: "javascript",
        label: "Javascript"
      }, {
        value: "jsx",
        label: "JSX"
      }, {
        value: "sql",
        label: "SQL"
      }, {
        value: "css",
        label: "CSS"
      }, {
        value: "html",
        label: "HTML"
      }, {
        value: "smarty",
        label: "Smarty"
      }, {
        value: "solidity",
        label: "Solidity"
      }, {
        value: "bash",
        label: "Bash"
      }];

      var chips = [];

      langs.map(function (item) {
        var className = [classes.chip, "test-chip"];

        if (item.value == _this2.state.lang) {
          className.push("active");
        }

        chips.push(_react2.default.createElement(_Chip2.default, {
          key: item.value,
          label: item.label,
          className: className.join(" ")
          // labelClassName="label"
          , onClick: function onClick() {
            _this2.setState({
              lang: item.value
            });
          }
        }));
      });

      editPanel = _react2.default.createElement(
        'div',
        { className: 'Editor-panel' },
        _react2.default.createElement(
          'div',
          { className: classes.row },
          chips
        ),
        _react2.default.createElement('textarea', {
          className: 'Editor-textValue',
          onChange: this._onValueChange,
          ref: 'textarea',
          value: this.state.textValue
        }),
        _react2.default.createElement(
          'div',
          { className: 'Editor-buttons' },
          _react2.default.createElement(
            'button',
            {
              className: buttonClass,
              disabled: this.state.invalidTeX,
              onClick: this._save
            },
            'Сохранить'
          ),
          _react2.default.createElement(
            'button',
            { className: 'Editor-removeButton', onClick: this._remove },
            '\u0423\u0434\u0430\u043B\u0438\u0442\u044C'
          )
        )
      );
    }

    return _react2.default.createElement(
      'div',
      { className: className },
      _react2.default.createElement(_CodeBlock2.default, {
        content: texContent,
        lang: this.state.lang,
        onClick: this._onClick
      }),
      editPanel
    );
  };

  return TextBlock;
}(_react2.default.Component), _class.defaultProps = {
  lang: "php"
}, _temp);
TextBlock.propTypes = process.env.NODE_ENV !== "production" ? {
  lang: _propTypes2.default.string.isRequired
} : {};
exports.default = (0, _withStyles2.default)(styles)(TextBlock);